/* Hirschmann Antenna Rotator Controller
 * 
 * Version 1.2
 *
 * 
 *
 * @author: Giovanni Carboni IZ5PQT
 * @date: 28 February 2020
 */
#include <LiquidCrystal.h>
#include <EEPROM.h>

// define pins
int motorCoilA = 9;
int motorCoilB = 8;
int currentPin = 1;  // Ammeter input
int motorPin = 3;    // Motor Start 
int zeroPin = 2;     // Zero Rotator
int upCntr = 10;     // UP Toggle switch
int dnCntr = 11;     // DOWN Toggle switch
int Ireadpin = 14;   // Current readout analog in

float Ival = 0;
unsigned long trot;
// Istall is the threshold current to determine the stall condition
float Istall = 0.75 ;
// TT is the motor period in ms
unsigned long TT = 82000;

int delayTime = 100;
int idir = 1;  // CW direction
// reset counters
int ns = 0 ;
int nstep = 0;
int nn = 0; 
int ndir = 0 ;
int ndirIni = 0;   // this is the current direction in degrees
int ndirFin = 0;
int nread = 0;

float K1 = 0.185 ; // ACS712 A/V conversion factor
float K2 = 3.3  ;  // Rectifier gain R3/R1 = 33k / 10k
float K3 = 1024./5. ; // Arduino ADC coonversion factor
float pival = 3.1416 ;
float CC = K1*K2*K3*sqrt(2)/pival ;

char gra = 223;  // degree symbol
char lt =  60;   // < symbol
char gt =  62;   // > symbol
// define greek delta symbol
byte Delta[] = {
  B00100,
  B00100,
  B01010,
  B01010,
  B10001,
  B10001,
  B11111,
  B00000
};

unsigned long previousMillis = 0;

LiquidCrystal lcd(12,13,7,6,5,4);

void setup() {
  lcd.begin(16,2);
  analogReference(DEFAULT);
  pinMode(motorCoilA, OUTPUT);
  pinMode(motorCoilB, OUTPUT);
  pinMode(upCntr, INPUT);
  pinMode(dnCntr, INPUT);
  pinMode(motorPin, INPUT);
  pinMode(zeroPin, INPUT);
  pinMode(Ireadpin, INPUT);
// enable pullup resistors on toggle switch inputs
  digitalWrite(upCntr, HIGH) ;
  digitalWrite(dnCntr, HIGH) ;
  digitalWrite(motorPin, HIGH);
  digitalWrite(zeroPin, HIGH); 
  
  Serial.begin(9600);
//  Serial.println("SETUP"); 
  lcd.clear();
  delay(2000);
  idir = 2;  // CCW
// create Delta
  lcd.createChar(0,Delta);
  lcd.home();
//  lcd.print("**** IZ5PQT ****");
//  lcd.print(" ");
  lcd.print(lt);lcd.print(lt);lcd.print(lt);
  lcd.print("  IZ5PQT  ");
  lcd.print(gt);lcd.print(gt);lcd.print(gt);
  lcd.setCursor(0,1);
  lcd.print(lt);
  lcd.print("MYROTOR  V.1.2");
  lcd.print(gt);
  delay(4000);
  digitalWrite(motorCoilA, LOW); 
// reads position stored in EEPROM
//    EEPROM.write(0,0);  // for debug
//    EEPROM.write(1,0);
  ndirIni = EEPROM.read(0)+256*EEPROM.read(1);
  lcd.clear();
 
  lcd.print("Memory pos. ");
  lcd.print(ndirIni);
  lcd.print(gra);
  delay(1000);
// if memorized position is zero then go to zero degrees
  if (ndirIni == 0) {
  lcd.clear();
  lcd.print("    ZEROING  ");
  delay(500);
  Zero();
  }
  ndir = ndirIni;
 }


void loop() {
 
      nstep = 0;
      lcd.clear();
      lcd.print("Actual pos. ");
//      lcd.print(ndir);
      lcd.print(ndirIni);
      lcd.print(gra);
      lcd.setCursor(0,1);
//      delay(1000);
      nstep = 0;
// at beginning of the loop ndir = ndirIni
// at the end ndir is the target direction    
      while(digitalRead(upCntr) == 0 && ndir < 360) {
        idir = 1;
        ndir++ ;
        lcd.clear();
        lcd.setCursor(0,1);
        lcd.print("    Go to ");
        lcd.print(ndir);
        lcd.print(gra);
        delay(100);
      }
      while(digitalRead(dnCntr) == 0 && ndir > 0) {
        idir = 2;
        ndir-- ;
        lcd.clear(); 
        lcd.setCursor(0,1);
        lcd.print("    Go to ");
        lcd.print(ndir);
        lcd.print(gra);
        Serial.println(ndir);
        delay(100);
      }
      delay(2000);
// Computes the displacement 
      nstep = ndir - ndirIni;  // nstep is Delta
      if(nstep > 0) idir = 1;
      if(nstep < 0) idir = 2;
// computes the required rotation time      
      trot = abs(nstep)*TT/360;
      lcd.clear();
      lcd.print("Actual pos. ");
      lcd.print(ndirIni);
      lcd.print(gra);
      lcd.setCursor(0,1);
      lcd.write(byte(0));
      lcd.print(" = ");
      lcd.print(nstep);
      lcd.print(gra);
      delay(1000);
  
//  checks if calibration (zeroing) is requested 
  if(digitalRead(zeroPin) == 0) {
    lcd.clear();
    lcd.print("    ZEROING  ");
    delay(1000);
    Zero();
    ndir = 0;
    ndirIni = 0;
// writes position to memory
    EEPROM.write(0,lowByte(ndirIni));
    EEPROM.write(1,highByte(ndirIni));
    nstep = 0;
  }
// checks if motor should move to target position
   if(digitalRead(motorPin) == 0 && idir == 1) {
    lcd.clear();
    lcd.print("*** Rotating ***");
    lcd.setCursor(0,1);
    lcd.print("    Trot = ");
    lcd.print(trot/1000);
    
    CWrot(trot);
    ndirIni = ndir;  
    lcd.clear();
    lcd.print("Position saved");
// writes final position to memory
    EEPROM.write(0,lowByte(ndirIni));
    EEPROM.write(1,highByte(ndirIni));
 //   lcd.print(ndirIni);
    delay(2000);
//   Serial.print("ndirIni salvata ");
//   Serial.println(ndirIni); 
   }
   if(digitalRead(motorPin) == 0 && idir == 2) {
    lcd.clear();
    lcd.print("*** Rotating ***");
    lcd.setCursor(0,1);
    lcd.print("    Trot = ");
    lcd.print(trot/1000);
   
    CCWrot(trot);
    ndirIni = ndir;  
    lcd.clear();
    lcd.print("Position saved");
// writes final position to memory    
    EEPROM.write(0,lowByte(ndirIni));
    EEPROM.write(1,highByte(ndirIni));
    delay(2000);
//    Serial.print("ndirIni salvata ");
//    Serial.println(ndirIni); 
   }
   
} 
void CWrot(unsigned long time) 
// clockwise rotation 
 { 
  digitalWrite(motorCoilA, HIGH);
  delay(time);
  digitalWrite(motorCoilA, LOW);
 }
void CCWrot(unsigned long time)
// counterclockwise rotation  
 { 
  digitalWrite(motorCoilB, HIGH);
  delay(time);
  digitalWrite(motorCoilB, LOW);
 }  
float Aread()
 {
// reads the motor current via the ACS712 chip
  float result;
  uint32_t start = millis();
  
  int mm = 0;
  float Vmed = 0;
  int readVal = 0;
  while ((millis() - start) < 200) // average over 200 ms
  {
    readVal = analogRead(Ireadpin);
    Vmed =  Vmed + float(readVal);
    mm++;
  }
// 328 is the pedestal correponding to 1.6 V when current is zero
// 1.6 V is the bias voltage on the LM358 non inverting input
    result = (Vmed/float(mm)-328.)/CC ;
    return result;  
}
  
void Zero() {
// zeroes rotor by CCW rotation until stall 
// stall detected by drop in current drawn on terminal 3

 digitalWrite(motorCoilB, HIGH); 
 nn = 0;
 lcd.clear();
 while(nn < 999) {
 // lcd.print(nn);
    delay(100);
    
    lcd.clear();
    lcd.print(nn); 
    if(nn > 9) lcd.print("   I= ");
    else lcd.print("   I= ");
  
    Ival = Aread();
    lcd.print(Ival,4);
    lcd.print(" A");
    delay(100);
 // stall threshold=Istall (normally 0.75 A)
 // requires nn = 5 readings before stop
    if(Ival < Istall   && nn > 5 && Ival > 0) goto stall ;
    Serial.println(Ival);
    nn++;
  }
  stall:
// stall detected
  delay(500);
  digitalWrite(motorCoilB, LOW); 
  digitalWrite(motorCoilA, LOW);     
  lcd.setCursor(0,1);
  lcd.print("                ");
  lcd.setCursor(0,0);
  lcd.print("                ");
  
  lcd.setCursor(0,0);
  lcd.print("   END STOP ");
  lcd.setCursor(0,1);
  lcd.print("***  READY  ***");
  delay(1000);

}  
