/* Hirschmann Antenna Rotator Controller
 * 
 * Setup program
 * Version 1.0
 *
 * 
 *
 * @author: Giovanni Carboni IZ5PQT
 * @date: 28 February 2020
 */
#include <LiquidCrystal.h>

// define Arduino pins
int motorCoilA = 9;
int motorCoilB = 8;
int currentPin = 1;  // Ammeter input
int motorPin = 3;    // Motor Start 
int zeroPin = 2;     // Zero Rotator
int upCntr = 10;     // UP Toggle switch
int dnCntr = 11;     // DOWN Toggle switch
int Ireadpin = 14;   // Current readout analog in

float Ival = 0;
int delayTime = 100;

// constants for current conversion

float K1 = 0.185 ; // ACS712 A/V conversion factor
float K2 = 3.3  ;  // Rectifier gain R3/R1 = 33k / 10k
float K3 = 1024./5. ; // Arduino ADC coonversion factor
float pival = 3.1416 ;
float CC = K1*K2*K3*sqrt(2)/pival ;

char gra = 223;  // degree symbol
char lt =  60;   // < symbol
char gt =  62;   // > symbol

// define arrow symbols
byte dnarrow[] = {
  B00100,
  B00100,
  B00100,
  B00100,
  B00100,
  B10101,
  B01110,
  B00100
};
byte uparrow[] = {
  B00100,
  B01110,
  B10101,
  B00100,
  B00100,
  B00100,
  B00100,
  B00100
};

unsigned long previousMillis = 0;

LiquidCrystal lcd(12,13,7,6,5,4);

void setup() {
  lcd.begin(16,2);
  analogReference(DEFAULT);
  pinMode(motorCoilA, OUTPUT);
  pinMode(motorCoilB, OUTPUT);
  pinMode(upCntr, INPUT);
  pinMode(dnCntr, INPUT);
  pinMode(motorPin, INPUT);
  pinMode(zeroPin, INPUT);
  pinMode(Ireadpin, INPUT);
// enable pullup resistors on toggle switch inputs
  digitalWrite(upCntr, HIGH) ;
  digitalWrite(dnCntr, HIGH) ;
  digitalWrite(motorPin, HIGH);
  digitalWrite(zeroPin, HIGH); 
  
  Serial.begin(9600);
//  Serial.println("SETUP"); 
  lcd.clear();
  delay(2000);
 // create arrow symbols
  lcd.createChar(0,dnarrow);
  lcd.createChar(1,uparrow);
  lcd.home();

  lcd.print(lt);lcd.print(lt);lcd.print(lt);
  lcd.print("  IZ5PQT  ");
  lcd.print(gt);lcd.print(gt);lcd.print(gt);
  lcd.setCursor(0,1);
  lcd.print(lt);
  lcd.print("MYROTOR  SETUP");
  lcd.print(gt);
  delay(4000);
 }


void loop() {
      lcd.clear();
      lcd.print("  ");
      lcd.write(byte(1));
      lcd.print("  CW  turn");
      lcd.setCursor(0,1);
      lcd.print("  ");
      lcd.write(byte(0));
      lcd.print("  CCW turn");

      uint32_t tzero = millis();
// CW rotation      
      while(digitalRead(upCntr) == 0) {
        digitalWrite(motorCoilA, HIGH);
        delay(200);
        lcd.clear();
        
        lcd.print("  I = ");
        Ival = Aread();
        lcd.print(Ival,4);
        lcd.print(" A");
        lcd.setCursor(0,1);
        lcd.print("  T = ");
        lcd.print((millis()-tzero)*0.001);
        lcd.print(" s");
      }
      digitalWrite(motorCoilA, LOW);
      delay(1000);
// CCW rotation 
      tzero = millis();  
    while(digitalRead(dnCntr) == 0) {
        digitalWrite(motorCoilB, HIGH);

        delay(200);
        lcd.clear();
        
        lcd.print("  I = ");
        Ival = Aread();
        lcd.print(Ival,4);
        lcd.print(" A");
        lcd.setCursor(0,1);
        lcd.print("  T = ");
        lcd.print((millis()-tzero)*0.001);
        lcd.print(" s");
       
      }
      digitalWrite(motorCoilB, LOW);
      delay(1000);
} 
 
float Aread()
 {
// reads the motor current via the ACS712 chip
  float result;
  uint32_t start = millis();
  
  int mm = 0;
  float Vmed = 0;
  int readVal = 0;
  while ((millis() - start) < 200) // average over 200 ms
  {
    readVal = analogRead(Ireadpin);
    Vmed =  Vmed + float(readVal);
    mm++;
  }
// 328 is the pedestal correponding to 1.6 V when current is zero
// 1.6 V is the bias voltage on the LM358 non inverting input
    result = (Vmed/float(mm)-328.)/CC ;
    return result;  
}
  
